/*
AntiCamper for RocketCTFBoost

This program is free software; you can redistribute and/or modify
it under the terms of the Open Unreal Mod License version 1.1.
*/
class AntiCamper extends Mutator config(RocketCTFBoost);

var config bool bShakeView;

var config float timeRange;
var config float minDistance;
var float endCamping;

var float oldMinDistance;
var float oldTimeRange;

var bool init;

var struct Tracking
{
	var Pawn p;
	var struct DataPoint
	{
		var vector location;
		var bool bWater;
	} DPoints[4];

	var byte index;
	var bool bCampingState;
	var bool bStartsMoving;
} Players[32];

function PostBeginPlay()
{
	if(init)
		return;
		
	init = True;
	SetTimer(timeRange / 4, True);
	
	oldMinDistance = minDistance;
	oldTimeRange = timeRange;
	
	endCamping = minDistance - minDistance / 4;
}

function int GetIndex(Pawn _p)
{
	local int i, e;
	
	e = -1;
	
	for(i = 0;i < 32;i++)
	{
		if(_p == Players[i].p)
			return i;
		else if(e == -1 && Players[i].p == None)
			e = i;
	}
	
	//init
	Players[e].p = _p;
	
	ResetLocations(e, _p.Location);
	
	return e;
}

function Timer()
{
	local Pawn p;
	local PlayerPawn pp;
	local int index;

	if(Level.Game.bGameEnded)
	{
		SetTimer(0.0, False);
		return;
	}
		
	//react on changes
	if(oldMinDistance != minDistance)
	{
		oldMinDistance = minDistance;
		endCamping = minDistance - minDistance / 4;
	}
	
	if(oldTimeRange != timeRange)
	{
		oldTimeRange = timeRange;
		SetTimer(timeRange / 4, True);
	}
		
	for(p = Level.PawnList; p != None; p = p.NextPawn)
	{
		pp = PlayerPawn(p);
		//typing or spec-> skip this data point
		if(!p.bIsPlayer || 
		p.bHidden ||
		p.PlayerReplicationInfo.bWaitingPlayer ||
		(pp != None && pp.bIsTyping))
			continue;
		
		//reset dmg for fc if needed; no camp detect
		if(p.PlayerReplicationInfo.HasFlag != None)
		{
			if(p.DamageScaling == 0)
				ResetDamageScaling(pp, p, True);
			continue;
		}
	
		//find
		index = GetIndex(p);
		
		//add data
		Players[index].DPoints[Players[index].index].location = p.Location;
		Players[index].DPoints[Players[index].index].bWater = p.FootRegion.Zone.bWaterZone;
			
		Players[index].index ++;
		if(Players[index].index == 4)
			Players[index].index = 0;
			
		if(!Players[index].bCampingState)
		{
			//still showing good movements
			if(GoodDistance(index, 0, 3, minDistance) ||
				GoodDistance(index, 0, 2, minDistance) ||
				GoodDistance(index, 0, 1, minDistance) ||
				GoodDistance(index, 1, 3, minDistance) ||
				GoodDistance(index, 1, 2, minDistance) ||
				GoodDistance(index, 2, 3, minDistance))
			{}
			else //didn't move enough -> camper
			{

				Players[index].bCampingState = True;
				Players[index].bStartsMoving = False;
				p.DamageScaling = 0;
				if(pp != None)
				{
					p.ReceiveLocalizedMessage(class'CamperMessage', 0);
					if(bShakeView)
						p.ShakeView(2, -600, -5);
				}
			}
		}
		else
		{
			if(DoesFinishCamping(index))//just moved enough to end camping?
			{
				if(!Players[index].bStartsMoving)
					Players[index].bStartsMoving = True;//1st step
				else //2nd step = END of camping
				{
					Players[index].bCampingState = False;
					Players[index].bStartsMoving = False;
					ResetDamageScaling(pp, p, False);
				}
			}
			else
				Players[index].bStartsMoving = False; // fall back to all camping
				
		}
	}
}

function bool DoesFinishCamping(int index)
{
	switch(Players[index].index)
	{
		case 0://32
			return GoodDistance(index, 3, 2, endCamping);
		case 1://03
			return GoodDistance(index, 0, 3, endCamping);
		case 2://10
			return GoodDistance(index, 1, 0, endCamping);
		case 3://21
			return GoodDistance(index, 2, 1, endCamping);
	}
	return False;
}

function ResetDamageScaling(PlayerPawn pp, Pawn p, bool bNowFC)
{
	local Inventory inv;
	
	inv = p.FindInventoryType(class'UDamage');
	if(inv != None)
	{
		p.DamageScaling = 3;
		if(pp != None)
		{
			if(!bNowFC)
				p.ReceiveLocalizedMessage(class'CamperMessage', 2);
			else
				p.ReceiveLocalizedMessage(class'CamperMessage', 4);
		}
	}
	else
	{
		p.DamageScaling = p.default.DamageScaling;
		if(pp != None)
		{
			if(!bNowFC)
				p.ReceiveLocalizedMessage(class'CamperMessage', 1);
			else
				p.ReceiveLocalizedMessage(class'CamperMessage', 3);
		}
	}
	
	//set: no camping for new fcs
	if(bNowFC)
		Players[GetIndex(p)].bCampingState = False;
	
}

function PlayerSpawning(Pawn Other)
{
	ResetLocations(GetIndex(Other), Other.Location);
	Other.ReceiveLocalizedMessage(class'CamperMessage', 5);
}


//init a new or spawning player
function ResetLocations(int index, vector startLocation)
{
	Players[index].DPoints[0].location = startLocation;
	Players[index].DPoints[0].bWater = False;
	Players[index].DPoints[1].location = startLocation;
	Players[index].DPoints[1].bWater = False;
	Players[index].DPoints[2].location = startLocation;
	Players[index].DPoints[2].bWater = False;
	Players[index].DPoints[3].location = startLocation;
	Players[index].DPoints[3].bWater = False;
	
	Players[index].DPoints[3].location.z += minDistance + 1;
	Players[index].index = 1;
	Players[index].bCampingState = False;
	Players[index].bStartsMoving = False;
}

//consider slower speed in water 
function bool GoodDistance(int index, int v1, int v2, int limit)
{
	if(Players[index].DPoints[v1].bWater || Players[index].DPoints[v2].bWater)
		return VSize(Players[index].DPoints[v1].location - Players[index].DPoints[v2].location)*2 > limit;
	else
		return VSize(Players[index].DPoints[v1].location - Players[index].DPoints[v2].location) > limit;
}


defaultproperties
{
    timeRange=4
    minDistance=250
}
