/*
RCTFArena for RocketCTFBoost

This program is free software; you can redistribute and/or modify
it under the terms of the Open Unreal Mod License version 1.1.
*/

class RCTFArena extends Arena config(RocketCTFBoost);

var bool postInit;

var Pawn playerWithBlueFlag;
var Pawn playerWithRedFlag;
var() config bool bAntiCamper;
var AntiCamper ac;

var config float EDIT_BOOSTING_DISTANCE;
var config float EDIT_BOOSTING_HEIGHT;

// Called after the game start
function PostBeginPlay(){
	
	if (postInit)
		return;
	postInit = True;

	// Register our MutatorTakeDamage
	Level.Game.RegisterDamageMutator(Self);
	if(bAntiCamper)
		ac = Spawn(class'AntiCamper');
}



function bool CheckReplacement(Actor Other, out byte bSuperRelevant)
{
	
	if(Other.class == class'UT_Jumpboots') //No jump boots
        return False;
	else if(Other.class == class'UT_Invisibility') //No invisibility
        return False;
	else
		return Super.CheckReplacement(Other, bSuperRelevant);
}

function MutatorTakeDamage( out int ActualDamage, Pawn Victim, Pawn InstigatedBy, out Vector HitLocation, out Vector Momentum, name DamageType)
{
	
	if(Victim == None || InstigatedBy == None) return;
		
	if(ActualDamage > 5000) return; //For maps that force the player to take damage itself (space in dm-hyperblast)

	if(InstigatedBy.bIsPlayer &&
		Victim.bIsPlayer)
	{
		//boosting
		if(InstigatedBy == Victim)
		{
			ActualDamage = 0; //No damage
			//FC
			if(Victim.IsA('TournamentPlayer') && CTFFlag(Victim.PlayerReplicationInfo.HasFlag) != None)
			{
				//manipulate boosting power
				Momentum.x *= EDIT_BOOSTING_DISTANCE;
				Momentum.y *= EDIT_BOOSTING_DISTANCE;
				Momentum.z *= EDIT_BOOSTING_HEIGHT;
			}
		}
		else //shooting
		{
			if(InstigatedBy.PlayerReplicationInfo.Team == Victim.PlayerReplicationInfo.Team)
			{
				Momentum = Vect(0,0,0); // No pushing
			}
		}
	}
	
	if(NextDamageMutator != None ) //Execute the same method of a next mutator if it exists
        NextDamageMutator.MutatorTakeDamage(ActualDamage, Victim, InstigatedBy, HitLocation, Momentum, DamageType);
}

function Tick(float d)
{
	local Pawn P;
	for (P = Level.PawnList; P != None; P = P.NextPawn)
	{
		if(playerWithBlueFlag != P && P.PlayerReplicationInfo.hasFlag != None && P.PlayerReplicationInfo.Team == 1)
		{
			playerWithBlueFlag = P;
			P.ReceiveLocalizedMessage(class'MessagePlusPlus', 0);
		}
		else if(playerWithRedFlag != P && P.PlayerReplicationInfo.hasFlag != None && P.PlayerReplicationInfo.Team == 0)
		{
			playerWithRedFlag = P;
			P.ReceiveLocalizedMessage(class'MessagePlusPlus', 0);
		}
		else if(playerWithBlueFlag == P && P.PlayerReplicationInfo.hasFlag == None) //if the saved doesn't have a flag now
		{
			playerWithBlueFlag = None;
			P.ReceiveLocalizedMessage(class'MessagePlusPlus', 1);
		}
		else if(playerWithRedFlag == P && P.PlayerReplicationInfo.hasFlag == None)
		{
			playerWithRedFlag = None;
			P.ReceiveLocalizedMessage(class'MessagePlusPlus', 1);
		}
	}
}


function ModifyPlayer(Pawn Other)
{
	if ( NextMutator != None )
		NextMutator.ModifyPlayer(Other);

	if(ac != None && !Other.PlayerReplicationInfo.bWaitingPlayer)
	{
		ac.PlayerSpawning(Other);
	}
}

defaultproperties
{
    bAntiCamper=True
    EDIT_BOOSTING_DISTANCE=0.50
    EDIT_BOOSTING_HEIGHT=0.70
    WeaponName=WeightyRocket
    AmmoName=HeavyRocketPack
    WeaponString="RCTF_PlayerPack2.WeightyRocket"
    AmmoString="RCTF_PlayerPack2.HeavyRocketPack"
    DefaultWeapon=Class'RCTF_PlayerPack2.WeightyRocket'
}
